==> james-project-3.7.3-source-release.zip: james-project/event-sourcing/event-sourcing-core/src/main/scala/org/apache/james/eventsourcing/EventBus.scala <==
 /***************************************************************
 * Licensed to the Apache Software Foundation (ASF) under one   *
 * or more contributor license agreements.  See the NOTICE file *
 * distributed with this work for additional information        *
 * regarding copyright ownership.  The ASF licenses this file   *
 * to you under the Apache License, Version 2.0 (the            *
 * "License"); you may not use this file except in compliance   *
 * with the License.  You may obtain a copy of the License at   *
 *                                                              *
 *   http://www.apache.org/licenses/LICENSE-2.0                 *
 *                                                              *
 * Unless required by applicable law or agreed to in writing,   *
 * software distributed under the License is distributed on an  *
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY       *
 * KIND, either express or implied.  See the License for the    *
 * specific language governing permissions and limitations      *
 * under the License.                                           *
 ****************************************************************/
package org.apache.james.eventsourcing

import javax.inject.Inject
import org.apache.james.eventsourcing.eventstore.{EventStore, EventStoreFailedException}
import org.reactivestreams.Publisher
import org.slf4j.LoggerFactory
import reactor.core.scala.publisher.{SFlux, SMono}

object EventBus {
  private val LOGGER = LoggerFactory.getLogger(classOf[EventBus])
}

class EventBus @Inject() (eventStore: EventStore, subscribers: Set[Subscriber]) {
  @throws[EventStoreFailedException]
  def publish(events: Iterable[Event]): SMono[Void] =
    SMono(eventStore.appendAll(events))
        .`then`(runHandlers(events))

  private def runHandlers(events: Iterable[Event]): SMono[Void] =
    SFlux.fromIterable(events.flatMap((event: Event) => subscribers.map(subscriber => (event, subscriber))))
      .concatMap(infos => runHandler(infos._1, infos._2))
      .`then`()
      .`then`(SMono.empty)

  private def runHandler(event: Event, subscriber: Subscriber): Publisher[Void] =
    SMono(ReactiveSubscriber.asReactiveSubscriber(subscriber).handleReactive(event))
      .onErrorResume(e => {
        EventBus.LOGGER.error("Error while calling {} for {}", subscriber, event, e)
        SMono.empty
      })
}

==> james-project-3.8.0-source-release.zip: james-project/event-sourcing/event-sourcing-core/src/main/scala/org/apache/james/eventsourcing/EventBus.scala <==
 /***************************************************************
 * Licensed to the Apache Software Foundation (ASF) under one   *
 * or more contributor license agreements.  See the NOTICE file *
 * distributed with this work for additional information        *
 * regarding copyright ownership.  The ASF licenses this file   *
 * to you under the Apache License, Version 2.0 (the            *
 * "License"); you may not use this file except in compliance   *
 * with the License.  You may obtain a copy of the License at   *
 *                                                              *
 *   http://www.apache.org/licenses/LICENSE-2.0                 *
 *                                                              *
 * Unless required by applicable law or agreed to in writing,   *
 * software distributed under the License is distributed on an  *
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY       *
 * KIND, either express or implied.  See the License for the    *
 * specific language governing permissions and limitations      *
 * under the License.                                           *
 ****************************************************************/
package org.apache.james.eventsourcing

import javax.inject.Inject
import org.apache.james.eventsourcing.eventstore.{EventStore, EventStoreFailedException}
import org.reactivestreams.Publisher
import org.slf4j.LoggerFactory
import reactor.core.scala.publisher.{SFlux, SMono}

object EventBus {
  private val LOGGER = LoggerFactory.getLogger(classOf[EventBus])
}

class EventBus @Inject() (eventStore: EventStore, subscribers: Set[Subscriber]) {
  @throws[EventStoreFailedException]
  def publish(events: Iterable[EventWithState]): SMono[Void] =
    SMono(eventStore.appendAll(events.map(_.event)))
        .`then`(runHandlers(events))

  private def runHandlers(events: Iterable[EventWithState]): SMono[Void] =
    SFlux.fromIterable(events.flatMap((commandExecuted: EventWithState) => subscribers.map(subscriber => (commandExecuted, subscriber))))
      .concatMap(infos => runHandler(infos._1, infos._2))
      .`then`()
      .`then`(SMono.empty)

  private def runHandler(commandExecuted: EventWithState, subscriber: Subscriber): Publisher[Void] =
    SMono(ReactiveSubscriber.asReactiveSubscriber(subscriber).handleReactive(commandExecuted))
      .onErrorResume(e => {
        EventBus.LOGGER.error("Error while calling {} for {}", subscriber, commandExecuted, e)
        SMono.empty
      })
}
